import pytest
from streamlit import session_state
from aic_core.streamlit.page import AICPage, app_state


def test_app_state_decorator():
    # Clear session state before test
    session_state.clear()

    # Create a test class to decorate
    @app_state("test_path")
    class TestClass:
        def __init__(self, value=None):
            self.value = value

    # First instance creation
    instance1 = TestClass(value="test")
    assert instance1.value == "test"
    assert "test_path" in session_state
    assert session_state["test_path"] is instance1

    # Second call should return the same instance
    instance2 = TestClass(value="different")
    assert instance2 is instance1
    assert instance2.value == "test"  # Should keep original value

    # Test with different path
    @app_state("other_path")
    class OtherClass:
        def __init__(self, value=None):
            self.value = value

    # Should create new instance for different path
    other_instance = OtherClass(value="other")
    assert "other_path" in session_state
    assert other_instance is not instance1
    assert other_instance.value == "other"


class TestAICPage(AICPage):
    def run(self) -> None:
        pass


def test_aic_page_abstract():
    # Should be able to instantiate concrete implementation
    page = TestAICPage()
    assert isinstance(page, AICPage)

    # Should not be able to instantiate abstract class
    with pytest.raises(TypeError):
        AICPage()



"""Shared components."""

import streamlit as st
from aic_core.agent.agent_hub import AgentHub


class ToolSelectorMixin:
    """Tool selector mixin. A tool is a function or a Pydantic model."""

    def list_function_names(self, repo_id: str) -> list[str]:
        """List function names from Agents Hub."""
        hf_repo = AgentHub(repo_id)
        return hf_repo.list_files(AgentHub.tools_dir)

    def list_result_type_names(self, repo_id: str) -> list[str]:
        """List result type names from Agents Hub."""
        hf_repo = AgentHub(repo_id)
        return hf_repo.list_files(AgentHub.result_types_dir)

    def _list_tool_output_names(self, repo_id: str) -> list[str]:
        """List tool or output names from Agents Hub."""
        return self.list_function_names(repo_id) + self.list_result_type_names(repo_id)

    def tool_selector(self, repo_id: str) -> str:
        """Tool selector."""
        # Get tool names from Agents Hub
        tool_names = self._list_tool_output_names(repo_id)
        selected_tool_names = st.sidebar.multiselect(
            "Function or Pydantic model name",
            tool_names,
            max_selections=1,
        )
        selected_tool = selected_tool_names[0] if selected_tool_names else ""
        return selected_tool


class AgentSelectorMixin:
    """Agent selector mixin."""

    def list_agent_names(self, repo_id: str) -> list[str]:
        """List all agents."""
        hf_repo = AgentHub(repo_id)
        return hf_repo.list_files(AgentHub.agents_dir)

    def agent_selector(self, repo_id: str) -> str:
        """Agent selector."""
        agent_names = self.list_agent_names(repo_id)
        return st.sidebar.selectbox("Agent", agent_names)



"""Feedly MCP server."""

import httpx
from mcp.server.fastmcp import FastMCP
from aic_core.logging import get_logger
from feedly.api_client.session import FeedlySession
from feedly.api_client.stream import StreamOptions


server = FastMCP("Feedly MCP")
logger = get_logger(__name__)


@server.tool()
def get_feedly_news(
    max_count: int = 10, category: str = "AI", feedly_token: str | None = None
) -> list[dict]:
    """Get the latest news from Feedly.

    Args:
        max_count: The maximum number of news to get.
        category: The category of the news to get.
        feedly_token: The Feedly token to use. If not provided, the token will
            be fetched from the environment.
    """
    session = FeedlySession(auth=feedly_token)
    category_name = session.user.user_categories.get(category)
    results = []

    for article in category_name.stream_contents(
        options=StreamOptions(max_count=max_count)
    ):
        results.append(
            {
                "title": article["title"],
                "id": article["id"],
            }
        )

    return results


@server.tool()
def read_uninteresting(entry_id: str, feedly_token: str | None = None) -> None:
    """Mark an uninteresting entry as read.

    Args:
        entry_id: The id of the entry to mark as read.
        feedly_token: The Feedly token to use. If not provided, the token will
            be fetched from the environment.
    """
    session = FeedlySession(auth=feedly_token)
    access_token = session.auth.auth_token
    url = "https://cloud.feedly.com/v3/markers"

    headers = {
        "Authorization": f"OAuth {access_token}",
        "Content-Type": "application/json",
    }

    payload = {"action": "markAsRead", "type": "entries", "entryIds": [entry_id]}

    response = httpx.post(url, json=payload, headers=headers)

    if response.status_code == 200:
        logger.info(f"Marking entry {entry_id} as read.")
    else:
        logger.error(
            f"Failed to mark entry as read. Status code: {response.status_code}"
        )



"""Logging module."""

import logging


logging.basicConfig(
    format="%(asctime)s - %(name)s - %(levelname)s - %(message)s", level=logging.INFO
)


def get_logger(name: str) -> logging.Logger:
    """Get a logger."""
    logger = logging.getLogger(name)
    return logger



import json
from unittest.mock import patch
import pytest
from pydantic_ai.messages import ToolCallPart
from aic_core.agent.result_types import (
    Choice,
    ComponentRegistry,
    NumberInput,
    TableOutput,
    TextInput,
    TextOutput,
)


@pytest.fixture
def mock_streamlit():
    """Fixture to mock streamlit components."""
    with (
        patch("streamlit.text_input") as mock_text_input,
        patch("streamlit.number_input") as mock_number_input,
        patch("streamlit.radio") as mock_radio,
        patch("streamlit.multiselect") as mock_multiselect,
        patch("streamlit.text") as mock_text,
        patch("streamlit.markdown") as mock_markdown,
        patch("streamlit.latex") as mock_latex,
        patch("streamlit.json") as mock_json,
        patch("streamlit.table") as mock_table,
    ):
        # Setup return values for mocks
        mock_text_input.return_value = "test input"
        mock_number_input.return_value = 42
        mock_radio.return_value = "option1"
        mock_multiselect.return_value = ["option1", "option2"]
        mock_text.return_value = None
        mock_markdown.return_value = None
        mock_latex.return_value = None
        mock_json.return_value = None
        mock_table.return_value = None

        yield {
            "text_input": mock_text_input,
            "number_input": mock_number_input,
            "radio": mock_radio,
            "multiselect": mock_multiselect,
            "text": mock_text,
            "markdown": mock_markdown,
            "latex": mock_latex,
            "json": mock_json,
            "table": mock_table,
        }


def test_generate_text_input(mock_streamlit):
    """Test generating a text input component."""
    params = TextInput(
        type="text_input",
        label="Test Input",
        key="test_key",
        help="Test help",
        user_input="initial value",
    )
    part = ToolCallPart(
        tool_name="final_result_TextInput",
        args=json.dumps(params.model_dump()),
        tool_call_id="call_test",
        part_kind="tool-call",
    )

    ComponentRegistry.generate_st_component(part)

    mock_streamlit["text_input"].assert_called_once()
    call_args = mock_streamlit["text_input"].call_args[1]
    assert call_args["label"] == "Test Input"
    assert call_args["key"] == "test_key"
    assert call_args["help"] == "Test help"
    assert call_args["value"] == "initial value"


def test_generate_number_input(mock_streamlit):
    """Test generating a number input component."""
    params = NumberInput(
        type="number_input",
        label="Test Number",
        key="test_key",
        min_value=0,
        max_value=100,
        step=1,
        user_input=42,
    )
    part = ToolCallPart(
        tool_name="final_result_NumberInput",
        args=json.dumps(params.model_dump()),
        tool_call_id="call_test",
        part_kind="tool-call",
    )

    ComponentRegistry.generate_st_component(part)

    mock_streamlit["number_input"].assert_called_once()
    call_args = mock_streamlit["number_input"].call_args[1]
    assert call_args["label"] == "Test Number"
    assert call_args["key"] == "test_key"
    assert call_args["min_value"] == 0
    assert call_args["max_value"] == 100
    assert call_args["step"] == 1
    assert call_args["value"] == 42


def test_generate_radio(mock_streamlit):
    """Test generating a radio component."""
    params = Choice(
        type="radio",
        label="Test Radio",
        key="test_key",
        options=["option1", "option2", "option3"],
        user_input="option1",
    )

    part = ToolCallPart(
        tool_name="final_result_Choice",
        args=json.dumps(params.model_dump()),
        tool_call_id="call_test",
        part_kind="tool-call",
    )
    ComponentRegistry.generate_st_component(part)

    mock_streamlit["radio"].assert_called_once()
    call_args = mock_streamlit["radio"].call_args[1]
    assert call_args["label"] == "Test Radio"
    assert call_args["key"] == "test_key"
    assert call_args["options"] == ["option1", "option2", "option3"]
    assert call_args["index"] == 0  # index of "option1"


def test_generate_multiselect(mock_streamlit):
    """Test generating a multiselect component."""
    params = Choice(
        type="multiselect",
        label="Test Multiselect",
        key="test_key",
        options=["option1", "option2", "option3"],
        user_input=["option1", "option2"],
    )

    part = ToolCallPart(
        tool_name="final_result_Choice",
        args=json.dumps(params.model_dump()),
        tool_call_id="call_test",
        part_kind="tool-call",
    )
    ComponentRegistry.generate_st_component(part)

    mock_streamlit["multiselect"].assert_called_once()
    call_args = mock_streamlit["multiselect"].call_args[1]
    assert call_args["label"] == "Test Multiselect"
    assert call_args["key"] == "test_key"
    assert call_args["options"] == ["option1", "option2", "option3"]
    assert call_args["default"] == ["option1", "option2"]


def test_generate_latex_output(mock_streamlit):
    """Test generating a latex output component."""
    params = TextOutput(type="latex", body="Test output latex")

    part = ToolCallPart(
        tool_name="final_result_TextOutput",
        args=json.dumps(params.model_dump()),
        tool_call_id="call_test",
        part_kind="tool-call",
    )
    ComponentRegistry.generate_st_component(part)

    mock_streamlit["latex"].assert_called_once_with("Test output latex")


def test_generate_table_output(mock_streamlit):
    """Test generating a table output component."""
    params = TableOutput(
        type="table", data={"column1": [1, 2, 3], "column2": ["a", "b", "c"]}
    )

    part = ToolCallPart(
        tool_name="final_result_TableOutput",
        args=json.dumps(params.model_dump()),
        tool_call_id="call_test",
        part_kind="tool-call",
    )
    ComponentRegistry.generate_st_component(part)

    mock_streamlit["table"].assert_called_once()
    call_args = mock_streamlit["table"].call_args[1]
    assert call_args["data"] == {"column1": [1, 2, 3], "column2": ["a", "b", "c"]}


def test_generate_json_output(mock_streamlit):
    """Test generating a json output component."""
    params = TextOutput(
        type="json", body={"key": "value", "number": 42, "list": [1, 2, 3]}
    )

    part = ToolCallPart(
        tool_name="final_result_TextOutput",
        args=json.dumps(params.model_dump()),
        tool_call_id="call_test",
        part_kind="tool-call",
    )
    ComponentRegistry.generate_st_component(part)

    mock_streamlit["json"].assert_called_once_with(
        {"key": "value", "number": 42, "list": [1, 2, 3]}
    )


def test_contains_component():
    """Test checking if a component is registered."""
    # Test with registered component without prefix
    assert ComponentRegistry.contains_component("TextInput") is True
    assert ComponentRegistry.contains_component("NumberInput") is True
    assert ComponentRegistry.contains_component("Choice") is True
    assert ComponentRegistry.contains_component("TextOutput") is True
    assert ComponentRegistry.contains_component("TableOutput") is True

    # Test with registered component with prefix
    assert ComponentRegistry.contains_component("final_result_TextInput") is True
    assert ComponentRegistry.contains_component("final_result_NumberInput") is True
    assert ComponentRegistry.contains_component("final_result_Choice") is True
    assert ComponentRegistry.contains_component("final_result_TextOutput") is True
    assert ComponentRegistry.contains_component("final_result_TableOutput") is True

    # Test with unregistered component
    assert ComponentRegistry.contains_component("NonExistentComponent") is False
    assert (
        ComponentRegistry.contains_component("final_result_NonExistentComponent")
        is False
    )



import asyncio
from unittest.mock import AsyncMock, MagicMock, patch
import pytest
from pydantic_ai import Agent
from pydantic_ai.messages import (
    ModelRequest,
    TextPart,
    ToolCallPart,
    ToolReturnPart,
    UserPromptPart,
)
from aic_core.agent.agent import AgentConfig, AgentFactory
from aic_core.agent.result_types import ComponentRegistry
from aic_core.streamlit.agent_page import AgentPage, PageState


@pytest.fixture
def agent_page():
    return AgentPage(repo_id="test-repo", page_state=PageState())


@pytest.fixture
def mock_agent():
    agent = MagicMock(spec=Agent)
    agent._mcp_servers = []
    return agent


def test_init(agent_page):
    assert agent_page.repo_id == "test-repo"
    assert agent_page.page_title == "Agent"
    assert agent_page.user_role == "user"
    assert agent_page.assistant_role == "assistant"


def test_reset_chat_history(agent_page):
    agent_page.page_state.chat_history = ["some", "messages"]
    agent_page.reset_chat_history()
    assert agent_page.page_state.chat_history == []


def test_get_agent():
    # Setup
    agent_page = AgentPage(repo_id="test-repo", page_state=PageState())
    mock_agent = MagicMock(spec=Agent)
    mock_config = MagicMock(spec=AgentConfig)

    # Mock the from_hub method of AgentConfig
    with patch.object(
        AgentConfig, "from_hub", return_value=mock_config
    ) as mock_from_hub:
        # Mock the AgentFactory
        with patch.object(
            AgentFactory, "create_agent", return_value=mock_agent
        ) as mock_create_agent:
            # Call the method
            result = agent_page.get_agent("test-agent")

            # Verify the calls
            mock_from_hub.assert_called_once_with("test-repo", "test-agent")
            mock_create_agent.assert_called_once()

            # Verify the result
            assert result == mock_agent


def test_get_response_without_mcp_servers(agent_page, mock_agent):
    user_input = "Hello"
    mock_result = MagicMock()
    mock_result.new_messages.return_value = ["message1", "message2"]
    mock_agent.run = AsyncMock(return_value=mock_result)

    with patch("streamlit.chat_message") as mock_chat_message:
        asyncio.run(agent_page.get_response(user_input, mock_agent))

        mock_chat_message.assert_called_once()
        assert agent_page.page_state.chat_history == ["message1", "message2"]


def test_get_response_with_mcp_servers(agent_page, mock_agent):
    mock_agent._mcp_servers = ["server1"]
    user_input = "Hello"
    mock_result = MagicMock()
    mock_result.new_messages.return_value = ["message1"]
    mock_agent.run = AsyncMock(return_value=mock_result)
    mock_agent.run_mcp_servers = MagicMock()
    mock_agent.run_mcp_servers.return_value.__aenter__ = AsyncMock()
    mock_agent.run_mcp_servers.return_value.__aexit__ = AsyncMock()
    agent_page.reset_chat_history()

    with patch("streamlit.chat_message") as mock_chat_message:
        asyncio.run(agent_page.get_response(user_input, mock_agent))

        mock_chat_message.assert_called_once()
        assert agent_page.page_state.chat_history == ["message1"]


def test_display_chat_history(agent_page):
    message = ModelRequest(
        parts=[TextPart(content="Hello"), UserPromptPart(content="Hi")]
    )
    agent_page.page_state.chat_history = [message]

    with patch("streamlit.chat_message") as mock_chat_message:
        agent_page.display_chat_history()
        assert mock_chat_message.call_count == 2


@patch("streamlit.title")
@patch("streamlit.chat_input")
@patch("streamlit.sidebar.button")
def test_run(mock_button, mock_chat_input, mock_title, agent_page):
    mock_chat_input.return_value = None
    agent_page.agent_selector = MagicMock()
    agent_page.get_agent = MagicMock()

    with patch.object(agent_page, "display_chat_history") as mock_display_chat_history:
        agent_page.run()

        mock_title.assert_called_once_with("Agent")
        mock_button.assert_called_once_with(
            "Reset chat history", on_click=agent_page.reset_chat_history
        )
        mock_chat_input.assert_called_once_with("Enter a message")
        mock_display_chat_history.assert_called_once()


def test_display_parts(agent_page):
    """Test display_parts method with different message parts."""
    # Mock streamlit components
    mock_chat_message = MagicMock()
    mock_chat_message.return_value.write = MagicMock()

    # Test TextPart
    with patch("streamlit.chat_message", return_value=mock_chat_message):
        agent_page.display_parts([TextPart(content="Hello")], None)

    # Reset mock for next test
    mock_chat_message.reset_mock()

    # Test UserPromptPart
    with patch("streamlit.chat_message", return_value=mock_chat_message):
        agent_page.display_parts([UserPromptPart(content="Hi")], None)

    # Reset mock for next test
    mock_chat_message.reset_mock()

    # Test ToolCallPart with valid component
    with patch("streamlit.chat_message", return_value=mock_chat_message):
        with patch.object(ComponentRegistry, "contains_component", return_value=True):
            with patch.object(
                ComponentRegistry, "generate_st_component"
            ) as mock_generate:
                tool_call = ToolCallPart(
                    tool_name="test_tool",
                    args="{}",
                    tool_call_id="123",
                    part_kind="tool-call",
                )
                tool_return = ToolReturnPart(
                    tool_name="test_tool",
                    content="result",
                    tool_call_id="123",
                    part_kind="tool-return",
                )
                agent_page.display_parts([tool_call], tool_return)
                mock_generate.assert_called_once_with(tool_call, tool_return)

    # Reset mock for next test
    mock_chat_message.reset_mock()

    # Test ToolCallPart with invalid component
    with patch("streamlit.chat_message", return_value=mock_chat_message):
        with patch.object(ComponentRegistry, "contains_component", return_value=False):
            tool_call = ToolCallPart(
                tool_name="invalid_tool",
                args="{}",
                tool_call_id="123",
                part_kind="tool-call",
            )
            agent_page.display_parts([tool_call], None)



from unittest.mock import Mock, patch
import pytest
from aic_core.streamlit.mixins import AgentSelectorMixin, ToolSelectorMixin


@pytest.fixture
def tool_selector():
    return ToolSelectorMixin()


@pytest.fixture
def agent_selector():
    return AgentSelectorMixin()


@pytest.fixture
def mock_agent_hub():
    with patch("aic_core.streamlit.mixins.AgentHub") as mock:
        # Configure the mock to return predictable values
        mock_instance = Mock()
        mock.return_value = mock_instance
        yield mock_instance


def test_list_function_names(tool_selector, mock_agent_hub):
    mock_agent_hub.tools_dir = "tools"
    mock_agent_hub.list_files.return_value = ["function1.py", "function2.py"]

    result = tool_selector.list_function_names("test-repo")

    assert result == ["function1.py", "function2.py"]


def test_list_result_type_names(tool_selector, mock_agent_hub):
    mock_agent_hub.result_types_dir = "result_types"
    mock_agent_hub.list_files.return_value = ["type1.py", "type2.py"]

    result = tool_selector.list_result_type_names("test-repo")

    assert result == ["type1.py", "type2.py"]


def test_list_tool_output_names(tool_selector, mock_agent_hub):
    mock_agent_hub.tools_dir = "tools"
    mock_agent_hub.result_types_dir = "result_types"
    mock_agent_hub.list_files.side_effect = [
        ["function1.py", "function2.py"],
        ["type1.py", "type2.py"],
    ]

    result = tool_selector._list_tool_output_names("test-repo")

    assert result == ["function1.py", "function2.py", "type1.py", "type2.py"]


@patch("streamlit.sidebar.multiselect")
def test_tool_selector(mock_multiselect, tool_selector, mock_agent_hub):
    # Mock the tool names
    mock_agent_hub.list_files.side_effect = [["function1.py"], ["type1.py"]]

    # Mock the multiselect to return a selection
    mock_multiselect.return_value = ["function1.py"]

    result = tool_selector.tool_selector("test-repo")

    mock_multiselect.assert_called_once_with(
        "Function or Pydantic model name",
        ["function1.py", "type1.py"],
        max_selections=1,
    )
    assert result == "function1.py"


@patch("streamlit.sidebar.multiselect")
def test_tool_selector_no_selection(mock_multiselect, tool_selector, mock_agent_hub):
    # Mock the tool names
    mock_agent_hub.list_files.side_effect = [["function1.py"], ["type1.py"]]

    # Mock the multiselect to return no selection
    mock_multiselect.return_value = []

    result = tool_selector.tool_selector("test-repo")

    assert result == ""


def test_list_agent_names(agent_selector, mock_agent_hub):
    mock_agent_hub.agents_dir = "agents"
    mock_agent_hub.list_files.return_value = ["agent1.py", "agent2.py"]

    result = agent_selector.list_agent_names("test-repo")

    assert result == ["agent1.py", "agent2.py"]


@patch("streamlit.sidebar.selectbox")
def test_agent_selector(mock_selectbox, agent_selector, mock_agent_hub):
    mock_agent_hub.agents_dir = "agents"
    mock_agent_hub.list_files.return_value = ["agent1.py", "agent2.py"]
    mock_selectbox.return_value = "agent1.py"

    result = agent_selector.agent_selector("test-repo")

    mock_selectbox.assert_called_once_with("Agent", ["agent1.py", "agent2.py"])
    assert result == "agent1.py"



"""Hub module for hosting tools."""

import importlib
import json
import os
import time
from collections.abc import Callable
from types import ModuleType
from huggingface_hub import delete_file, hf_hub_download, snapshot_download, upload_file
from huggingface_hub.errors import LocalEntryNotFoundError
from pydantic import BaseModel


class AgentHub:
    """Hugging Face Repo class.

    This class provides an interface to interact with Hugging Face repositories.
    It allows loading files, configs, and tools from a specified repository,
    as well as handling local caching when possible.
    """

    tools_dir: str = "tools"
    agents_dir: str = "agents"
    result_types_dir: str = "result_types"
    repo_type: str = "space"
    update_interval: int = 600

    def __init__(self, repo_id: str) -> None:
        """Initialize the Hugging Face Hub."""
        self.repo_id = repo_id

    def _load_module(self, module_name: str, path: str) -> ModuleType:
        """Load a module from a path."""
        spec = importlib.util.spec_from_file_location(module_name, path)
        if spec is None or spec.loader is None:  # pragma: no cover
            raise ImportError(f"Could not load spec for module {path}")
        module = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(module)
        return module

    def _check_extension(self, filename: str, extension: str) -> str:
        """Check if the filename has the correct extension."""
        if not filename.endswith(extension):  # pragma: no cover
            filename = f"{filename}{extension}"
        return filename

    def _lazy_update(self) -> None:
        """Lazily update the local cache.

        Update the repo only when self.get_file_path is called, and when
        interval has passed.
        """
        cache_path = self.download_files(local_files_only=True)
        last_modified = os.path.getmtime(cache_path)
        if time.time() - last_modified > self.update_interval:
            self.download_files()
            os.utime(cache_path, None)

    def download_files(self, local_files_only: bool = False) -> str:
        """Download all files from the Hugging Face Hub.

        This should be called at the service start up, as well as when any
        changes are made to the repo.
        """
        return snapshot_download(
            repo_id=self.repo_id,
            repo_type=self.repo_type,
            local_files_only=local_files_only,
        )

    def get_file_path(self, filename: str, subdir: str) -> str:
        """Get the local path to a file in the repo."""
        self._lazy_update()
        match subdir:
            case self.tools_dir:
                extension = ".py"
                subfolder = self.tools_dir
            case self.result_types_dir:
                extension = ".py"
                subfolder = self.result_types_dir
            case self.agents_dir:
                extension = ".json"
                subfolder = self.agents_dir
            case _:  # pragma: no cover
                raise ValueError(f"Invalid subdir: {subdir}")

        filename = self._check_extension(filename, extension)
        try:
            file_path = hf_hub_download(
                repo_id=self.repo_id,
                filename=filename,
                subfolder=subfolder,
                local_files_only=True,
                repo_type=self.repo_type,
            )
        except LocalEntryNotFoundError:
            file_path = hf_hub_download(
                repo_id=self.repo_id,
                filename=filename,
                subfolder=subfolder,
                repo_type=self.repo_type,
            )
        return file_path

    def delete_file(self, filename: str, subdir: str) -> None:
        """Delete a file from the Hugging Face Hub."""
        delete_file(
            path_in_repo=f"{subdir}/{filename}",
            repo_id=self.repo_id,
            repo_type=self.repo_type,
        )

    def load_config(self, filename: str) -> dict:
        """Load a config from the Hugging Face Hub."""
        if not filename:  # pragma: no cover
            return {}
        file_path = self.get_file_path(filename, self.agents_dir)

        with open(file_path) as file:
            return json.load(file)

    def load_tool(self, filename: str) -> Callable | None:
        """Load a tool from the Hugging Face Hub."""
        if not filename:  # pragma: no cover
            return None
        name_without_extension = filename.split(".")[0]
        file_path = self.get_file_path(filename, self.tools_dir)
        module = self._load_module(name_without_extension, file_path)
        func = getattr(module, name_without_extension)
        assert callable(func)

        return func

    def load_result_type(self, filename: str) -> type[BaseModel] | None:
        """Load a result type from the Hugging Face Hub."""
        if not filename:  # pragma: no cover
            return None
        name_without_extension = filename.split(".")[0]
        file_path = self.get_file_path(filename, self.result_types_dir)
        module = self._load_module(name_without_extension, file_path)
        model = getattr(module, name_without_extension)
        assert isinstance(model, type) and issubclass(model, BaseModel)

        return model

    def upload_content(
        self,
        filename: str,
        content: str,
        subdir: str,
    ) -> None:
        """Upload a file to the Hugging Face Hub."""
        match subdir:
            case self.tools_dir:
                extension = ".py"
            case self.result_types_dir:
                extension = ".py"
            case self.agents_dir:
                extension = ".json"
            case _:
                raise ValueError(f"Invalid type: {subdir}")

        filename = self._check_extension(filename, extension)

        upload_file(
            path_or_fileobj=content.encode("utf-8"),
            path_in_repo=f"{subdir}/{filename}",
            repo_id=self.repo_id,
            repo_type=self.repo_type,
            commit_message=f"Update {filename}",
        )

    def list_files(self, subdir: str) -> list[str]:
        """List all files in the Hugging Face Hub."""
        repo_dir = snapshot_download(
            repo_id=self.repo_id, repo_type=self.repo_type, local_files_only=True
        )
        subdir_path = os.path.join(repo_dir, subdir)

        if not os.path.exists(subdir_path):
            return []

        if not os.path.isdir(subdir_path):
            return []

        return [
            f.split(".")[0]
            for f in os.listdir(subdir_path)
            if os.path.isfile(os.path.join(subdir_path, f))
        ]



"""Template for custom function or Pydantic model."""

from pydantic import BaseModel, Field


class MyModel(BaseModel):
    """My model."""

    attr1: str = Field(..., description="The first attribute.")
    attr2: int = Field(..., description="The second attribute.")


def my_function(arg1: int, arg2: int) -> int:
    """My function. Docstrings and arguments are useful for function calls.

    Args:
        arg1: The first argument.
        arg2: The second argument.
    """
    return arg1 + arg2



"""Streamlit page class."""

from abc import ABC, abstractmethod
from collections.abc import Callable
from typing import Any
from streamlit import session_state


def app_state(file_path: str) -> Callable:
    """Singleton decorator that takes a file path argument."""

    def decorator(cls: Callable) -> Callable:
        def wrapper(*args: Any, **kwargs: Any) -> Callable:
            if file_path not in session_state:
                session_state[file_path] = cls(*args, **kwargs)
            return session_state[file_path]

        return wrapper

    return decorator


class AICPage(ABC):
    """AIC page."""

    @abstractmethod
    def run(self) -> None:
        """Run the page."""
        pass  # pragma: no cover



"""Agent page."""

import asyncio
import streamlit as st
from pydantic_ai import Agent
from pydantic_ai.messages import (
    ModelMessage,
    ModelRequestPart,
    ModelResponsePart,
    TextPart,
    ToolCallPart,
    ToolReturnPart,
    UserPromptPart,
)
from aic_core.agent.agent import AgentConfig, AgentFactory
from aic_core.agent.result_types import ComponentRegistry
from aic_core.streamlit.mixins import AgentSelectorMixin
from aic_core.streamlit.page import AICPage


class PageState:
    """Page state.

    Acting as a template here. Must be defined in the page that uses it.
    """

    chat_history: list[ModelMessage] = []


class AgentPage(AICPage, AgentSelectorMixin):
    """Agent page.

    PageState needs to have the following values:
    - chat_history: list[ModelMessage]
    """

    def __init__(
        self, repo_id: str, page_state: PageState, page_title: str = "Agent"
    ) -> None:
        """Initialize the page."""
        super().__init__()
        self.repo_id = repo_id
        self.page_title = page_title
        self.page_state = page_state
        self.user_role = "user"
        self.assistant_role = "assistant"

    def reset_chat_history(self) -> None:
        """Reset chat history."""
        self.page_state.chat_history = []

    def get_agent(self, agent_name: str) -> Agent:
        """Get agent."""
        agent_config = AgentConfig.from_hub(self.repo_id, agent_name)
        agent_factory = AgentFactory(agent_config)
        agent = agent_factory.create_agent()

        return agent

    async def get_response(self, user_input: str, agent: Agent) -> None:
        """Get response from agent."""
        history = self.page_state.chat_history
        st.chat_message(self.user_role).write(user_input)
        if agent._mcp_servers:
            async with agent.run_mcp_servers():
                result = await agent.run(user_input, message_history=history)  # type: ignore
        else:
            result = await agent.run(user_input, message_history=history)  # type: ignore
        self.page_state.chat_history.extend(result.new_messages())

    def display_parts(
        self,
        msg_parts: list[ModelRequestPart] | list[ModelResponsePart],
        next_msg_part: ModelRequestPart | ModelResponsePart | None,
    ) -> None:
        """Display message parts."""
        for part in msg_parts:
            match part:
                case TextPart():
                    st.chat_message(self.assistant_role).write(part.content)
                case UserPromptPart():
                    st.chat_message(self.user_role).write(part.content)
                case ToolCallPart():
                    if not ComponentRegistry.contains_component(part.tool_name):
                        return

                    assert isinstance(next_msg_part, ToolReturnPart)
                    with st.chat_message(self.assistant_role):
                        ComponentRegistry.generate_st_component(part, next_msg_part)
                case _:  # pragma: no cover
                    pass

    def display_chat_history(self) -> None:
        """Display chat history."""
        history_shifted = self.page_state.chat_history[1:] + [None]
        for msg, next_msg in zip(
            self.page_state.chat_history, history_shifted, strict=False
        ):
            self.display_parts(msg.parts, None if not next_msg else next_msg.parts[0])

    def run(self) -> None:
        """Run the page."""
        st.title(self.page_title)
        self.display_chat_history()

        agent_name = self.agent_selector(self.repo_id)
        agent = self.get_agent(agent_name)
        st.sidebar.button("Reset chat history", on_click=self.reset_chat_history)
        user_input = st.chat_input("Enter a message")

        if user_input:  # pragma: no cover
            asyncio.run(self.get_response(user_input, agent))
            st.rerun()



"""Agent config page."""

from typing import get_args
import streamlit as st
from code_editor import code_editor
from pydantic_ai.models import KnownModelName
from aic_core.agent.agent import AgentConfig
from aic_core.agent.agent_hub import AgentHub
from aic_core.agent.result_types import ComponentRegistry
from aic_core.streamlit.mixins import AgentSelectorMixin, ToolSelectorMixin
from aic_core.streamlit.page import AICPage


class AgentConfigPage(AICPage, AgentSelectorMixin, ToolSelectorMixin):
    """Agent config page."""

    def __init__(self, repo_id: str) -> None:
        """Initialize the page."""
        super().__init__()
        self.repo_id = repo_id

    def list_result_type_options(self) -> dict[str, str]:
        """List all result types."""
        python_types = {type: f"{type} (P)" for type in ["str", "int", "float", "bool"]}
        internal_types = {
            type: f"{type} (I)"
            for type in ComponentRegistry.get_registered_components()
        }
        external_types = {
            type: f"{type} (E)" for type in self.list_result_type_names(self.repo_id)
        }
        return {**python_types, **internal_types, **external_types}

    def configure(self, config: AgentConfig) -> AgentConfig:
        """Widgets to configure the agent."""
        model_options = [
            model
            for model in get_args(KnownModelName.__value__)
            if model.startswith("openai")
        ]
        model = st.selectbox(
            "Select a model",
            model_options,
            index=model_options.index(config.model),
        )

        result_type_options = self.list_result_type_options()
        result_type = st.multiselect(
            "Result type (**P**: Python, **I**: Internal, **E**: External)",
            options=result_type_options,
            format_func=lambda x: result_type_options[x],
            default=config.result_type,
        )
        st.write("**System prompt**")
        system_prompt = code_editor(
            config.system_prompt,
            lang="markdown",
            response_mode="debounce",
            options={"wrap": True},
        )["text"]
        temperature = st.slider(
            "Temperature",
            min_value=0.0,
            max_value=2.0,
            value=config.model_settings.get("temperature", 1.0)
            if config.model_settings
            else 1.0,
        )
        top_p = st.slider(
            "Top P",
            min_value=0.0,
            max_value=1.0,
            value=config.model_settings.get("top_p", 1.0)
            if config.model_settings
            else 1.0,
        )
        model_settings = {
            "temperature": temperature,
            "top_p": top_p,
        }
        retries = st.number_input(
            "Retries", min_value=0, max_value=100, value=config.retries
        )
        result_tool_name = st.text_input(
            "Result tool name", value=config.result_tool_name
        )
        result_tool_description = st.text_input(
            "Result tool description", value=config.result_tool_description
        )
        result_retries = st.number_input(
            "Result retries", min_value=0, max_value=100, value=config.retries
        )
        list_known_tools = self.list_function_names(self.repo_id)
        default_known_tools = [
            tool for tool in config.known_tools if tool in list_known_tools
        ]
        known_tools = st.multiselect(
            "Known tools", options=list_known_tools, default=default_known_tools
        )
        hf_tools = st.text_area("HF tools", value="\n".join(config.hf_tools))
        mcp_servers = st.text_area("MCP servers", value="\n".join(config.mcp_servers))
        defer_model_check = st.toggle(
            "Defer model check", value=config.defer_model_check
        )
        end_strategy = st.selectbox("End strategy", ["early", "exhaustive"])
        instrument = st.toggle("Instrument", value=config.instrument)
        name = st.text_input("Name", value=config.name)
        name = name.replace(" ", "_")

        return AgentConfig(
            model=model,
            result_type=result_type,
            system_prompt=system_prompt or config.system_prompt,
            model_settings=model_settings,
            retries=retries,
            result_tool_name=result_tool_name,
            result_tool_description=result_tool_description,
            result_retries=result_retries,
            known_tools=known_tools,
            hf_tools=[x for x in hf_tools.split("\n") if x],
            mcp_servers=[x for x in mcp_servers.split("\n") if x],
            defer_model_check=defer_model_check,
            end_strategy=end_strategy,
            name=name,
            instrument=instrument,
            repo_id=self.repo_id,
        )

    def save_config(self, config: AgentConfig) -> None:
        """Save the config and trigger a re-download of the files."""
        config.push_to_hub()
        hf_repo = AgentHub(self.repo_id)
        hf_repo.download_files()

    def run(self) -> None:
        """Main function."""
        st.title("Custom Agent")
        agent_name = self.agent_selector(self.repo_id)
        if agent_name:
            config = AgentConfig.from_hub(self.repo_id, agent_name)
        else:  # Initialize a new agent
            config = AgentConfig(model="openai:gpt-4o", repo_id=self.repo_id)
        new_config = self.configure(config)
        if st.button(
            "Save", on_click=self.save_config, args=(new_config,)
        ):  # pragma: no cover
            st.success("Agent pushed to the hub.")



from unittest.mock import Mock, patch
import pytest
from aic_core.agent.agent import AgentConfig
from aic_core.agent.agent_hub import AgentHub
from aic_core.streamlit.agent_config import AgentConfigPage


@pytest.fixture
def mock_streamlit():
    """Mock streamlit components."""
    with (
        patch("aic_core.streamlit.agent_config.st") as mock_st,
        patch("aic_core.streamlit.agent_config.code_editor") as mock_code_editor,
    ):
        # Configure default mock returns for streamlit widgets
        mock_st.selectbox.return_value = "openai:gpt-4"
        mock_st.multiselect.return_value = ["str"]
        mock_code_editor.return_value = {"text": "test prompt"}
        mock_st.slider.return_value = 1.0
        mock_st.number_input.return_value = 3
        mock_st.text_input.return_value = "test_name"
        mock_st.toggle.return_value = False
        yield mock_st


@pytest.fixture
def agent_config_page():
    """Create a concrete AgentConfigPage instance for testing."""

    class ConcreteAgentConfigPage(AgentConfigPage):
        def re_download_files(self) -> None:
            pass

    return ConcreteAgentConfigPage("test-repo")


def test_configure(agent_config_page, mock_streamlit):
    """Test agent configuration."""
    initial_config = AgentConfig(model="openai:gpt-4", repo_id="test-repo")
    with (
        patch.object(
            agent_config_page, "list_result_type_names", return_value=["CustomType"]
        ),
        patch.object(
            agent_config_page,
            "list_function_names",
            return_value=["function1", "function2"],
        ),
    ):
        result = agent_config_page.configure(initial_config)

        assert isinstance(result, AgentConfig)
        assert result.model == "openai:gpt-4"
        assert result.result_type == ["str"]
        assert result.system_prompt == "test prompt"
        assert result.model_settings == {"temperature": 1.0, "top_p": 1.0}
        assert result.retries == 3
        assert result.name == "test_name"
        assert result.repo_id == "test-repo"


def test_save_config(agent_config_page):
    """Test saving configuration."""
    mock_config = Mock(spec=AgentConfig)
    with patch.object(AgentHub, "download_files"):
        agent_config_page.save_config(mock_config)

    mock_config.push_to_hub.assert_called_once()


def test_run(agent_config_page, mock_streamlit):
    """Test the run method."""
    with (
        patch.object(agent_config_page, "agent_selector", return_value="test_agent"),
        patch.object(AgentConfig, "from_hub") as mock_from_hub,
        patch.object(agent_config_page, "save_config"),
    ):
        mock_config = Mock(spec=AgentConfig)
        mock_from_hub.return_value = mock_config
        with patch.object(agent_config_page, "configure", return_value=mock_config):
            agent_config_page.run()

            mock_streamlit.title.assert_called_once_with("Custom Agent")
            agent_config_page.agent_selector.assert_called_once_with("test-repo")
            mock_from_hub.assert_called_once_with("test-repo", "test_agent")


def test_run_new_agent(agent_config_page, mock_streamlit):
    """Test the run method when creating a new agent."""
    with (
        patch.object(agent_config_page, "agent_selector", return_value=None),
        patch.object(agent_config_page, "save_config"),
    ):
        mock_config = Mock(spec=AgentConfig)
        with patch.object(agent_config_page, "configure", return_value=mock_config):
            agent_config_page.run()

            mock_streamlit.title.assert_called_once_with("Custom Agent")
            agent_config_page.agent_selector.assert_called_once_with("test-repo")



import pytest
from aic_core.streamlit.tool_template import MyModel, my_function


def test_my_function():
    # Test basic addition
    assert my_function(2, 3) == 5
    assert my_function(0, 0) == 0
    assert my_function(-1, 1) == 0

    # Test with larger numbers
    assert my_function(100, 200) == 300


def test_my_model():
    # Test valid model creation
    model = MyModel(attr1="test", attr2=42)
    assert model.attr1 == "test"
    assert model.attr2 == 42

    # Test model validation errors
    with pytest.raises(ValueError):
        MyModel(attr1=None, attr2=42)  # attr1 cannot be None

    with pytest.raises(ValueError):
        MyModel(attr1="test", attr2="not_an_integer")  # attr2 must be an integer



"""Tool config page."""

import importlib.util
import os
import sys
from types import ModuleType
import streamlit as st
from code_editor import code_editor
from huggingface_hub.errors import EntryNotFoundError
from pydantic import BaseModel
from aic_core.agent.agent_hub import AgentHub
from aic_core.streamlit.mixins import ToolSelectorMixin
from aic_core.streamlit.page import AICPage


class ToolConfigPage(AICPage, ToolSelectorMixin):
    """Tool config page."""

    def __init__(self, repo_id: str) -> None:
        """Initialize the page."""
        super().__init__()
        self.repo_id = repo_id

    def load_code_as_module(
        self, code_text: str, module_name: str = "dynamic_module"
    ) -> ModuleType:
        """Load code text as a Python module.

        Args:
            code_text (str): The Python code as a string
            module_name (str): Name to give the module

        Returns:
            module: The loaded Python module
        """
        # Create a module spec
        spec = importlib.util.spec_from_loader(module_name, loader=None)
        assert spec

        # Create a new module based on the spec
        module = importlib.util.module_from_spec(spec)

        # Add the module to sys.modules
        sys.modules[module_name] = module

        # Execute the code within the module's namespace
        exec(code_text, module.__dict__)

        return module

    def save_tool(self, tool_name: str, code: str) -> None:
        """Save tool."""
        hf_repo = AgentHub(self.repo_id)
        try:
            # Test loading the code as a module
            dynamic_module = self.load_code_as_module(code)
            module_contents = [
                item for item in dir(dynamic_module) if not item.startswith("__")
            ]
            assert tool_name in module_contents
            func = getattr(dynamic_module, tool_name)
            # Upload the code to Tools Hub
            if isinstance(func, type) and issubclass(func, BaseModel):
                hf_repo.upload_content(tool_name, code, AgentHub.result_types_dir)
            else:
                hf_repo.upload_content(tool_name, code, AgentHub.tools_dir)
        except AssertionError:
            st.error(f"Definition `{tool_name}` not found in module")
            st.stop()
        except Exception as e:  # pragma: no cover
            st.error(f"Error loading code as module: {str(e)}")
            st.stop()
        hf_repo.download_files()

    def edit_tool(self, tool_name: str) -> None:
        """Edit tool."""
        hf_repo = AgentHub(self.repo_id)
        if tool_name:
            try:
                file_path = hf_repo.get_file_path(tool_name, AgentHub.tools_dir)
            except EntryNotFoundError:
                file_path = hf_repo.get_file_path(tool_name, AgentHub.result_types_dir)
            with open(file_path) as f:
                default_code = f.read()
        else:
            template_path = os.path.join(os.path.dirname(__file__), "tool_template.py")
            with open(template_path) as f:
                default_code = f.read()

        code = code_editor(
            default_code,
            lang="python",
            response_mode="debounce",
            options={"wrap": True},
        )

        tool_name = st.text_input(
            "Tool name",
            value=tool_name,
            disabled=not code["text"],
        )

        button_disabled = not tool_name or not code["text"]
        if st.button(
            "Save",
            disabled=button_disabled,
            on_click=self.save_tool,
            args=(tool_name, code["text"]),
        ):  # pragma: no cover
            st.success(f"Tool `{tool_name}` saved successfully!")

    def run(self) -> None:
        """Main function."""
        st.title("Custom Function or Pydantic Model")
        selected_tool = self.tool_selector(self.repo_id)
        self.edit_tool(selected_tool)



"""Streamlit MCP server with Pydantic models."""

from __future__ import annotations
import json
from collections.abc import Callable
from typing import Any, Literal
import streamlit as st
from pydantic import BaseModel
from pydantic_ai.messages import ToolCallPart, ToolReturnPart


class ComponentRegistry:
    """Registry for Streamlit component models."""

    _registry: dict[str, type[InputComponent | OutputComponent]] = {}

    @classmethod
    def register(cls) -> Callable:
        """Decorator to register a component model.

        Returns:
            A decorator function that registers the component model using its
            class name as the key
        """

        def decorator(
            component_class: type[InputComponent | OutputComponent],
        ) -> type[InputComponent | OutputComponent]:
            cls._registry[component_class.__name__] = component_class
            return component_class

        return decorator

    @classmethod
    def get_component_class(
        cls, component_name: str
    ) -> type[InputComponent | OutputComponent]:
        """Get a component class by its name.

        Args:
            component_name: The name of the component class

        Returns:
            The component class

        Raises:
            KeyError: If the component is not registered
        """
        if component_name not in cls._registry:  # pragma: no cover
            raise KeyError(f"Component '{component_name}' is not registered")
        return cls._registry[component_name]

    @classmethod
    def get_registered_components(cls) -> list[str]:
        """Get a list of all registered component names.

        Returns:
            A list of registered component class names
        """
        return list(cls._registry.keys())

    @classmethod
    def contains_component(cls, tool_name: str) -> bool:
        """Check if a component is registered."""
        return tool_name.replace("final_result_", "") in cls._registry

    @classmethod
    def generate_st_component(
        cls,
        tool_call_part: ToolCallPart,
        tool_return_part: ToolReturnPart | None = None,
    ) -> Any:
        """Generate a component based on the parameters."""

        def _input_callback(key: str) -> None:  # pragma: no cover
            value = st.session_state[key]
            updated_args = tool_call_part.args_as_dict()
            updated_args.update({"user_input": value})
            tool_call_part.args = (
                updated_args
                if isinstance(tool_call_part.args, dict)
                else json.dumps(updated_args)
            )
            if tool_return_part:  # pragma: no cover
                tool_return_part.content = f"User input: {value}"

        class_name = tool_call_part.tool_name.replace("final_result_", "")
        model = ComponentRegistry.get_component_class(class_name)
        params = model.model_validate(tool_call_part.args_as_dict())

        comp_type = params.type
        comp_func = getattr(st, comp_type)
        kwargs = params.model_dump(exclude={"type"})
        value = kwargs.pop("user_input", None)
        key = kwargs.get("key", None)
        match comp_type:
            case "text_input" | "text_area" | "number_input" | "slider":
                output = comp_func(
                    **kwargs, value=value, on_change=_input_callback, args=(key,)
                )
            case "radio":
                try:
                    index = kwargs["options"].index(value)
                except ValueError:  # pragma: no cover
                    index = None
                output = comp_func(
                    **kwargs, index=index, on_change=_input_callback, args=(key,)
                )
            case "multiselect":
                output = comp_func(
                    **kwargs,
                    default=value,
                    on_change=_input_callback,
                    args=(key,),
                )
            case "latex":
                output = comp_func(kwargs["body"])
            case "json":
                output = comp_func(kwargs["body"])
            case _:
                output = comp_func(**kwargs)
        return output


class StreamlitComponent(BaseModel, use_attribute_docstrings=True):
    """Parameters for components."""

    type: str
    """Streamlit component type."""


class InputComponent(StreamlitComponent):
    """Parameters for input components."""

    label: str
    """Label for the component."""
    key: str
    """Unique key for the component."""
    help: str | None = None
    """Help text for the component."""
    user_input: Any | None = None
    """Value input by the user for the component."""


class OutputComponent(StreamlitComponent):
    """Parameters for output components."""

    pass


@ComponentRegistry.register()
class TextInput(InputComponent):
    """Parameters for text input components."""

    type: Literal["text_input", "text_area"]
    """Streamlit component type."""
    user_input: str = ""
    """Value input by the user for the component."""


@ComponentRegistry.register()
class NumberInput(InputComponent):
    """Parameters for number input components."""

    type: Literal["number_input", "slider"]
    """Streamlit component type."""
    min_value: int | float | None = None
    """Minimum value for the component."""
    max_value: int | float | None = None
    """Maximum value for the component."""
    step: int | float | None = None
    """Step for the component."""
    user_input: int | float | None = None
    """Value input by the user for the component."""


@ComponentRegistry.register()
class Choice(InputComponent):
    """Parameters for choice components."""

    type: Literal["radio", "multiselect"]
    """Streamlit component type."""
    options: list[str]
    """Options for the component."""
    user_input: int | str | list[str] | None = None
    """Value input by the user for the component."""


@ComponentRegistry.register()
class TextOutput(OutputComponent):
    """Parameters for text output components."""

    type: Literal["latex", "code", "json"]
    """Streamlit component type."""
    body: str | dict
    """Body for the component."""
    language: str = "python"
    """Language for the component."""


@ComponentRegistry.register()
class TableOutput(OutputComponent):
    """Parameters for table output components."""

    type: Literal["table", "dataframe"]
    """Streamlit component type."""
    data: dict
    """Data dict for the component."""



from unittest.mock import Mock, mock_open, patch
import pytest
from huggingface_hub.errors import EntryNotFoundError
from aic_core.agent.agent_hub import AgentHub
from aic_core.streamlit.tool_config import ToolConfigPage


@pytest.fixture
def tool_config():
    class TestToolConfigPage(ToolConfigPage):
        def re_download_files(self):
            pass

    return TestToolConfigPage("test-repo")


def test_load_code_as_module(tool_config):
    code = """
def test_function():
    return "Hello"

class TestClass:
    pass
"""
    module = tool_config.load_code_as_module(code)
    assert hasattr(module, "test_function")
    assert hasattr(module, "TestClass")
    assert module.test_function() == "Hello"


@pytest.mark.parametrize(
    "tool_name,code,is_model",
    [
        ("test_tool", "def test_tool(): pass", False),
        (
            "TestModel",
            "from pydantic import BaseModel\nclass TestModel(BaseModel):\n    "
            "field: str",
            True,
        ),
    ],
)
def test_save_tool(tool_config, tool_name, code, is_model):
    mock_hub = Mock()
    with patch("aic_core.streamlit.tool_config.AgentHub", return_value=mock_hub):
        tool_config.save_tool(tool_name, code)


def test_save_tool_invalid_definition(tool_config):
    with (
        patch("streamlit.error") as mock_error,
        patch("streamlit.stop") as mock_stop,
        patch.object(AgentHub, "download_files"),
    ):
        tool_config.save_tool("missing_tool", "def other_tool(): pass")
        mock_error.assert_called_once_with(
            "Definition `missing_tool` not found in module"
        )
        mock_stop.assert_called_once()


def test_edit_tool_existing(tool_config):
    mock_content = "def existing_tool(): pass"
    mock_hub = Mock()
    mock_hub.get_file_path.return_value = "fake/path"

    with (
        patch("aic_core.streamlit.tool_config.AgentHub", return_value=mock_hub),
        patch("builtins.open", mock_open(read_data=mock_content)),
        patch("aic_core.streamlit.tool_config.code_editor") as mock_editor,
        patch("streamlit.text_input") as mock_input,
        patch("streamlit.button") as mock_button,
    ):
        mock_editor.return_value = {"text": mock_content}
        mock_input.return_value = "existing_tool"
        mock_button.return_value = True

        tool_config.edit_tool("existing_tool")
        mock_hub.get_file_path.assert_called_once()
        mock_editor.assert_called_once()
        mock_input.assert_called_once()

    # Case where EntryNotFoundError is raised
    mock_hub.get_file_path.side_effect = EntryNotFoundError
    with pytest.raises(EntryNotFoundError):
        tool_config.edit_tool("existing_tool")


def test_edit_tool_new(tool_config):
    mock_template = "# Template content"

    with (
        patch("builtins.open", mock_open(read_data=mock_template)),
        patch("aic_core.streamlit.tool_config.code_editor") as mock_editor,
        patch("streamlit.text_input") as mock_input,
        patch("streamlit.button") as mock_button,
    ):
        mock_editor.return_value = {"text": mock_template}
        mock_input.return_value = "new_tool"
        mock_button.return_value = True

        tool_config.edit_tool("")
        mock_editor.assert_called_once()
        mock_input.assert_called_once()


def test_run(tool_config):
    with (
        patch("streamlit.title") as mock_title,
        patch.object(
            tool_config, "tool_selector", return_value="test_tool"
        ) as mock_selector,
        patch.object(tool_config, "edit_tool") as mock_edit,
    ):
        tool_config.run()

        # Verify the title is set correctly
        mock_title.assert_called_once_with("Custom Function or Pydantic Model")

        # Verify tool_selector was called with repo_id
        mock_selector.assert_called_once_with(tool_config.repo_id)

        # Verify edit_tool was called with selected tool
        mock_edit.assert_called_once_with("test_tool")



from unittest.mock import Mock, patch
import pytest
from aic_core.mcp.feedly.server import get_feedly_news, read_uninteresting


@pytest.fixture
def mock_feedly_session():
    with patch("aic_core.mcp.feedly.server.FeedlySession") as mock_session:
        # Create mock article data
        mock_articles = [
            {"title": "Test Article 1", "id": "article1"},
            {"title": "Test Article 2", "id": "article2"},
        ]

        # Setup the mock chain
        mock_session_instance = Mock()
        mock_user = Mock()
        mock_category = Mock()

        mock_session.return_value = mock_session_instance
        mock_session_instance.user = mock_user
        mock_user.user_categories = {"AI": mock_category}
        mock_category.stream_contents.return_value = mock_articles

        mock_session_instance.auth.auth_token = "mock_token"
        yield mock_session


def test_get_feedly_news(mock_feedly_session):
    # Call the function
    results = get_feedly_news(max_count=2, category="AI", feedly_token="test_token")

    # Verify the results
    assert len(results) == 2
    assert results[0] == {"title": "Test Article 1", "id": "article1"}
    assert results[1] == {"title": "Test Article 2", "id": "article2"}

    # Verify the mock was called correctly
    mock_feedly_session.assert_called_once_with(auth="test_token")
    mock_session_instance = mock_feedly_session.return_value
    mock_category = mock_session_instance.user.user_categories["AI"]
    mock_category.stream_contents.assert_called_once()


def test_get_feedly_news_empty_results(mock_feedly_session):
    # Modify the mock to return empty results
    mock_session_instance = mock_feedly_session.return_value
    mock_category = mock_session_instance.user.user_categories["AI"]
    mock_category.stream_contents.return_value = []

    # Call the function
    results = get_feedly_news(max_count=2, category="AI", feedly_token="test_token")

    # Verify the results
    assert len(results) == 0


def test_read_uninteresting_success(mock_feedly_session):
    with patch("aic_core.mcp.feedly.server.httpx.post") as mock_post:
        # Setup mock response
        mock_post.return_value.status_code = 200

        # Call the function
        read_uninteresting(entry_id="test_entry_123", feedly_token="test_token")

        # Verify the POST request was made correctly
        mock_post.assert_called_once_with(
            "https://cloud.feedly.com/v3/markers",
            json={
                "action": "markAsRead",
                "type": "entries",
                "entryIds": ["test_entry_123"],
            },
            headers={
                "Authorization": "OAuth mock_token",
                "Content-Type": "application/json",
            },
        )


def test_read_uninteresting_failure():
    with patch("aic_core.mcp.feedly.server.httpx.post") as mock_post:
        # Setup mock response for failure case
        mock_post.return_value.status_code = 400

        # Call the function
        read_uninteresting(entry_id="test_entry_123", feedly_token="test_token")

        # Verify the POST request was made
        mock_post.assert_called_once()

        # The function should not raise an exception, but log an error
        # You might want to add logger verification if needed
